#include "Alignment.h"



void 
Overlap(vector<PharmacophorePoint>& P1, vector<PharmacophorePoint>& P2, vector<double>& Overlap_Scores, vector<double>& Volumes, std::map<int,int>& BestAlignment, int Opt, int sOpt){
  // Initialization
  double W1= 0.7, W2=(1-W1); // Weights for calculating Tversky scores
  double STanimoto = 0, CTanimoto = 0, TVshape = 0, TVchem=0;
  int npharm1=P1.size();
  int npharm2=P2.size();
  int npharm_max =2*MAX(npharm1, npharm2);
  if(npharm_max < 3)npharm_max=3;

  vector<PharmacophorePoint> QPharmacophores;
  vector<PharmacophorePoint> TPharmacophores;
  vector<PharmacophorePoint> TPharmacophores2;
  vector<PharmacophorePoint> TPharmacophores3;
  vector<PharmacophorePoint> TPharmacophoresB;
  std::map<int,int> Alignment;
  
  
  Coordinate P1Center, P2Center;
  double V1shape(0.0), V2shape(0.0), V1chem(0.0), V2chem(0.0), maxoverlap(0.00);
  double CordVec1[3], CordVec2[3];  
  double RMSD; double RM[3][3]={{0}}, T[3]={0}; double R[3][3]={{0}};  
  double **MAT, **VEC1, **VEC2, **SIM, **D1, **D2;
  double *U, *V;
  int    *Row,  *Col;
  
  CreateArray(&MAT, npharm_max, npharm_max);
  CreateArray(&SIM, npharm_max, npharm_max);
  CreateArray(&VEC1, npharm_max, 3); CreateArray(&VEC2, npharm_max, 3);
  CreateArray(&D1, npharm1, npharm1); CreateArray(&D2, npharm2, npharm2);
  U   = new double[npharm_max]; V   = new double[npharm_max];
  Row = new int   [npharm_max]; Col = new int [npharm_max];
  QPharmacophores= P1; 
  TPharmacophores= P2;
  
  // Precalculate Chemical Similarity
  for(int i=0; i<npharm_max;++i){
    for(int j=0; j<npharm_max;++j){
      SIM[i][j]=0;
      if(i >= npharm1){continue;}
      if(j >= npharm2){continue;}   
      if(((P1[i].func == 0) && (P2[j].func == 0)) || ((P1[i].func == 1) && (P2[j].func == 1)) ||
	 ((P1[i].func == 2) && ((P2[j].func == 2) || (P2[j].func == 3) || (P2[j].func == 4))) ||
	 ((P2[j].func == 2) && ((P1[i].func == 2) || (P1[i].func == 3) || (P1[i].func == 4))) ||
	 ((P1[i].func == P2[j].func) && (P1[i].func < 8)) ||
	 ((P1[i].alpha == P2[j].alpha) && ((P1[i].func <= 8) || (P2[j].func <= 8)))){
	SIM[i][j]  = 1.00;
      }
    }
  }
  
  // Calculate Molecular Gaussian Volume and Centroid
  P1Center.x =0; P1Center.y=0; P1Center.z=0;  
  for(int i=0; i <npharm1; ++i){       
    if(QPharmacophores[i].includedBefore){continue;}    
    double alpha1 = QPharmacophores[i].alpha;  
    double v1     = GCI * pow((PI/alpha1), 1.5);      
    QPharmacophores[i].vol=v1;
    V1shape     += v1;  
    V1chem      += v1;
    P1Center.x  += v1 * QPharmacophores[i].point.x;
    P1Center.y  += v1 * QPharmacophores[i].point.y;
    P1Center.z  += v1 * QPharmacophores[i].point.z;
    Coordinate C1 = QPharmacophores[i].point;
    for(int ii=0; ii <npharm1; ++ii){
      Coordinate C2 = QPharmacophores[ii].point;
      double D= dist(C1,C2);
      D1[i][ii]=D;
    }
  } 
  P1Center.x /= V1shape;
  P1Center.y /= V1shape;
  P1Center.z /= V1shape;

  double TM1[3][3]={{0}}, EVec1[3][3]={{0}}, EVal1[3]={0};
  for(int i=0; i < npharm1; i++){    
    QPharmacophores[i].point.x -= P1Center.x;
    QPharmacophores[i].point.y -= P1Center.y;
    QPharmacophores[i].point.z -= P1Center.z;    
    if(QPharmacophores[i].hasNormal){
      QPharmacophores[i].normal.x -= P1Center.x;
      QPharmacophores[i].normal.y -= P1Center.y;
      QPharmacophores[i].normal.z -= P1Center.z;
    }
    if(QPharmacophores[i].includedBefore){continue;}    
    double v1    = QPharmacophores[i].vol;
    TM1[0][0] += v1 * QPharmacophores[i].point.x * QPharmacophores[i].point.x;
    TM1[0][1] += v1 * QPharmacophores[i].point.x * QPharmacophores[i].point.y;
    TM1[0][2] += v1 * QPharmacophores[i].point.x * QPharmacophores[i].point.z;
    TM1[1][1] += v1 * QPharmacophores[i].point.y * QPharmacophores[i].point.y;
    TM1[1][2] += v1 * QPharmacophores[i].point.y * QPharmacophores[i].point.z;
    TM1[2][2] += v1 * QPharmacophores[i].point.z * QPharmacophores[i].point.z;
  }  
  TM1[1][0] = TM1[0][1];
  TM1[2][0] = TM1[0][2];
  TM1[2][1] = TM1[1][2];
  for(int k=0; k<3; k++)
    for(int l=0; l< 3; l++)
      TM1[k][l] /= V1shape;
  // Calculate Eigen Vectors
  Eigen(TM1, EVec1, EVal1);
  
  // Calculate Molecular Gaussian Volume and Centroid
  P2Center.x =0; P2Center.y=0; P2Center.z=0; 
  for(int j=0; j <npharm2; ++j){
    if(TPharmacophores[j].includedBefore){continue;}
    double alpha1 = TPharmacophores[j].alpha;
    double v1     = GCI * pow((PI/alpha1), 1.5);     
    TPharmacophores[j].vol=v1;
    V2chem      += v1;
    V2shape     += v1;
    P2Center.x  += v1 * TPharmacophores[j].point.x;
    P2Center.y  += v1 * TPharmacophores[j].point.y;
    P2Center.z  += v1 * TPharmacophores[j].point.z;  
    Coordinate C1 = TPharmacophores[j].point;
    for(int jj=0; jj <npharm2; ++jj){
      Coordinate C2 = TPharmacophores[jj].point;
      double D= dist(C1,C2);
      D2[j][jj]=D;
    }
  }   
  P2Center.x /= V2shape;
  P2Center.y /= V2shape;
  P2Center.z /= V2shape;
 
  double TM2[3][3]={{0}}, EVec2[3][3]={{0}}, EVal2[3]={0};
  for(int j=0; j < npharm2; j++){   
    TPharmacophores[j].point.x -= P2Center.x;
    TPharmacophores[j].point.y -= P2Center.y;
    TPharmacophores[j].point.z -= P2Center.z;
    if(TPharmacophores[j].hasNormal){
      TPharmacophores[j].normal.x -= P2Center.x;
      TPharmacophores[j].normal.y -= P2Center.y;
      TPharmacophores[j].normal.z -= P2Center.z;
    } 
    if(TPharmacophores[j].includedBefore){continue;}
    double v2    = TPharmacophores[j].vol;   
    TM2[0][0] += v2 * TPharmacophores[j].point.x * TPharmacophores[j].point.x;
    TM2[0][1] += v2 * TPharmacophores[j].point.x * TPharmacophores[j].point.y;
    TM2[0][2] += v2 * TPharmacophores[j].point.x * TPharmacophores[j].point.z;
    TM2[1][1] += v2 * TPharmacophores[j].point.y * TPharmacophores[j].point.y;
    TM2[1][2] += v2 * TPharmacophores[j].point.y * TPharmacophores[j].point.z;
    TM2[2][2] += v2 * TPharmacophores[j].point.z * TPharmacophores[j].point.z;    
  }  
  TM2[1][0] = TM2[0][1];
  TM2[2][0] = TM2[0][2];
  TM2[2][1] = TM2[1][2];
  for(int k=0; k<3; k++)
    for(int l=0; l< 3; l++)
      TM2[k][l] /= V2shape;
  // Calculate Eigen Vectors
  Eigen(TM2, EVec2, EVal2);
  
  // 1. Initial alignment based on all 3 Principle axis

  for(int pa1=0;pa1<3;++pa1){
    VEC1[pa1][0]=EVec1[pa1][0]; VEC1[pa1][1]=EVec1[pa1][1]; VEC1[pa1][2]=EVec1[pa1][2];    
  }
  for(int pa2=0;pa2<3;++pa2){
    VEC2[pa2][0]=EVec2[pa2][0]; VEC2[pa2][1]=EVec2[pa2][1]; VEC2[pa2][2]=EVec2[pa2][2];
  }
  Kabsch(VEC2, VEC1, 3, 1, &RMSD, T, R);  
  transformation(TPharmacophores,R,T);
  
  TPharmacophoresB=TPharmacophores;  
  

  
  
  int bestPAxis=0;;  
  double ShapeScore=0.0; double ChemScore=0;
  double BestShapeScore=0.0; double BestChemScore=0;
  double Delta=3.0; double maxR=0; int rotationStep=5;
  
  // 2. Check which axis is good 
  for(int pa1=0;pa1<3;++pa1){         
    CordVec1[0]=EVec1[pa1][0];
    CordVec1[1]=EVec1[pa1][1];
    CordVec1[2]=EVec1[pa1][2];       
    for(int pa2=0;pa2<3;++pa2){          
      CordVec2[0]=EVec2[pa2][0];
      CordVec2[1]=EVec2[pa2][1];
      CordVec2[2]=EVec2[pa2][2];           
      AlignVector(CordVec1, CordVec2, R);              
      TPharmacophores2=TPharmacophores; 
      transformation(TPharmacophores2,R,T);
      OverlapScore(QPharmacophores, TPharmacophores2, &ShapeScore, &ChemScore, MAT, SIM, U, V, Row, Col, VEC1, VEC2, Alignment, Opt);    
      double S = Score(ShapeScore,ChemScore,V1shape,V2shape,V1chem,V2chem,Opt);
     
      if(S > maxoverlap){
	bestPAxis        = pa2;
	TPharmacophoresB = TPharmacophores2;  
        maxoverlap       = S;
        BestShapeScore   = ShapeScore;
        BestChemScore    = ChemScore;
/*
        if(sOpt >0){
	  BestAlignment.clear();
	  for (std::map<int,int>::iterator it=Alignment.begin(); it!=Alignment.end(); ++it){
	    BestAlignment.insert(std::make_pair(it->first,it->second));	 
	  }
        }
*/
      } 
    }    
    for(int pa2=0;pa2<3;++pa2){          
      CordVec2[0]=-EVec2[pa2][0];
      CordVec2[1]=-EVec2[pa2][1];
      CordVec2[2]=-EVec2[pa2][2];           
      AlignVector(CordVec1, CordVec2, R);              
      TPharmacophores2=TPharmacophores; 
      transformation(TPharmacophores2,R,T);
      OverlapScore(QPharmacophores, TPharmacophores2, &ShapeScore, &ChemScore, MAT, SIM, U, V, Row, Col, VEC1, VEC2, Alignment,Opt);    
      double S = Score(ShapeScore, ChemScore, V1shape, V2shape, V1chem, V2chem, Opt);
      if(S > maxoverlap){
	bestPAxis        = pa2;
	TPharmacophoresB = TPharmacophores2;  
        maxoverlap       = S;
        BestShapeScore   = ShapeScore;
        BestChemScore    = ChemScore;
/*
        if(sOpt >0){
          BestAlignment.clear();
          for (std::map<int,int>::iterator it=Alignment.begin(); it!=Alignment.end(); ++it){
            BestAlignment.insert(std::make_pair(it->first,it->second));
          }
        }
*/
      }     
    }
    
  }
   
  // 3. Move around each axis
  TPharmacophores=TPharmacophoresB;
  maxR=maxoverlap;
  
  T[0]=0; T[1]=0; T[2]=0;
  
  for(int alphaR=0; alphaR<360; alphaR +=rotationStep){
    TPharmacophores2=TPharmacophores;
    RotationMatrix(RM, alphaR, EVec1[bestPAxis][0], EVec1[bestPAxis][1], EVec1[bestPAxis][2]);
    transformation(TPharmacophores2,RM,T);
    OverlapScore(QPharmacophores, TPharmacophores2, &ShapeScore, &ChemScore, MAT, SIM, U, V, Row, Col, VEC1, VEC2, Alignment,Opt);
    double S = Score(ShapeScore,ChemScore,V1shape,V2shape,V1chem,V2chem,Opt);

    double dE= (maxoverlap-S)*100;
    double r = rand(0,1);
    if((r < exp(-dE/15)) || (S > maxR)){
      maxR=S;
      if(S >maxoverlap){
	TPharmacophoresB = TPharmacophores2;  
	maxoverlap       = S;
	BestShapeScore   = ShapeScore;
	BestChemScore    = ChemScore;
/*
          if(sOpt >0){
            BestAlignment.clear();
            for (std::map<int,int>::iterator it=Alignment.begin(); it!=Alignment.end(); ++it){
              BestAlignment.insert(std::make_pair(it->first,it->second));
            }
          }
*/
      }
    }
    TPharmacophores2.clear();
  }
  TPharmacophores=TPharmacophoresB;
  maxR=maxoverlap;
  // 4. Align based on chemical groups
  for(int j=0; j<npharm2; ++j){
    int jj =j+1;
    int jjj=jj+1;
    if((jj >=npharm2) || (jjj>=npharm2)){continue;}
//  for(int jj=0; jj<npharm2;++jj){
    if(D2[j][jj]   < 5){continue;}
//  for(int jjj=0; jjj<npharm2;++jjj){
    if(D2[j][jjj]  < 5){continue;}
    if(D2[jj][jjj] < 5){continue;}
    
    for(int i=0; i<npharm1;++i){
      int ii = i+1;
      int iii= i+2;
      if((ii >=npharm1) || (iii >=npharm1)){continue;}
      if(D1[i][ii]   < 5){continue;}
      if(D1[i][iii]  < 5){continue;}
      if(D1[ii][iii] < 5){continue;}
//      if(abs(D1[i][ii]   - D2[j][jj])   > 3){continue;}
//      if(abs(D1[i][iii]  - D2[j][jjj])  > 3){continue;}
//      if(abs(D1[ii][iii] - D2[jj][jjj]) > 3){continue;}

      VEC1[0][0]=QPharmacophores[i].point.x;   VEC1[0][1]=QPharmacophores[i].point.y;   VEC1[0][2]=QPharmacophores[i].point.z;
      VEC1[1][0]=QPharmacophores[ii].point.x;  VEC1[1][1]=QPharmacophores[ii].point.y;  VEC1[1][2]=QPharmacophores[ii].point.z;
      VEC1[2][0]=QPharmacophores[iii].point.x; VEC1[2][1]=QPharmacophores[iii].point.y; VEC1[2][2]=QPharmacophores[iii].point.z;
      VEC2[0][0]=TPharmacophores[j].point.x;   VEC2[0][1]=TPharmacophores[j].point.y;   VEC2[0][2]=TPharmacophores[j].point.z;
      VEC2[1][0]=TPharmacophores[jj].point.x;  VEC2[1][1]=TPharmacophores[jj].point.y;  VEC2[1][2]=TPharmacophores[jj].point.z;
      VEC2[2][0]=TPharmacophores[jjj].point.x; VEC2[2][1]=TPharmacophores[jjj].point.y; VEC2[2][2]=TPharmacophores[jjj].point.z;

      Kabsch(VEC2, VEC1, 3, 1, &RMSD, T, R);
      if(RMSD >16){continue;}
      TPharmacophores2.clear();
      TPharmacophores2=TPharmacophores;
      transformation(TPharmacophores2,R,T); 
      OverlapScore(QPharmacophores, TPharmacophores2, &ShapeScore, &ChemScore, MAT, SIM, U, V, Row, Col, VEC1, VEC2, Alignment,Opt);
      double S = Score(ShapeScore,ChemScore,V1shape,V2shape,V1chem,V2chem,Opt); 
      if(S >maxoverlap){
	TPharmacophoresB=TPharmacophores2;
/*
	if(sOpt >0){
	  BestAlignment.clear();
	  for (std::map<int,int>::iterator it=Alignment.begin(); it!=Alignment.end(); ++it){
	    BestAlignment.insert(std::make_pair(it->first,it->second));
	  }
	}
*/
	maxoverlap= S;
	BestShapeScore = ShapeScore;
	BestChemScore  = ChemScore;
      }	 
    }
//  }      
//  }
  }
  TPharmacophores=TPharmacophoresB;
  maxR=maxoverlap; 

  // 5. Monte-Carlo simulation for improving alignment
  for(int randomStep=0; randomStep<100; ++randomStep){	         
    TPharmacophores3=TPharmacophores;
    double Tx=Delta*rand(-1,1);
    double Ty=Delta*rand(-1,1);
    double Tz=Delta*rand(-1,1);
    for(int k=0; k < npharm2; ++k){	  
      TPharmacophores3[k].point.x  += Tx;
      TPharmacophores3[k].point.y  += Ty;
      TPharmacophores3[k].point.z  += Tz;
      TPharmacophores3[k].normal.x += Tx;
      TPharmacophores3[k].normal.y += Ty;
      TPharmacophores3[k].normal.z += Tz;
    }          
    OverlapScore(QPharmacophores, TPharmacophores3, &ShapeScore, &ChemScore, MAT, SIM, U, V, Row, Col, VEC1, VEC2, Alignment,Opt);
    double S = Score(ShapeScore,ChemScore,V1shape,V2shape,V1chem,V2chem,Opt); 
    double dE= (maxoverlap-S)*100;
    double r2=rand(0,1);
    
    if((r2 < exp(-dE/15)) || (S > maxR)){
      maxR=S;      
      TPharmacophores2=TPharmacophores3;
      if(S >maxoverlap){
	TPharmacophoresB=TPharmacophores2;
	maxoverlap= S;
        BestShapeScore = ShapeScore;
        BestChemScore  = ChemScore;
/*
        if(sOpt >0){
          BestAlignment.clear();
          for (std::map<int,int>::iterator it=Alignment.begin(); it!=Alignment.end(); ++it){
            BestAlignment.insert(std::make_pair(it->first,it->second));
          }
        }
*/
      }
    }  
    TPharmacophores3.clear(); 
  }

  STanimoto = BestShapeScore/(V1shape + V2shape - BestShapeScore);
  CTanimoto = BestChemScore/(V1chem + V2chem - BestChemScore);
  TVshape   = BestShapeScore/( (W1*abs(V1shape-BestShapeScore)) + (W2*abs(V2shape-BestShapeScore)) + BestShapeScore);
  TVchem    = BestChemScore/( (W1*abs(V1chem-BestChemScore)) + (W2*abs(V2chem-BestChemScore)) + BestChemScore);

  Overlap_Scores[0]=STanimoto;
  Overlap_Scores[1]=CTanimoto;
  Overlap_Scores[2]=TVshape;
  Overlap_Scores[3]=TVchem;
  Overlap_Scores[4]=0;
  Volumes[0]=V1shape;
  Volumes[1]=V2shape;
  //	clear memory;
  
  QPharmacophores.clear();
  TPharmacophores.clear();
  TPharmacophores2.clear();
  TPharmacophores3.clear();
  TPharmacophoresB.clear();
  DeleteArray(&VEC1, npharm_max);
  DeleteArray(&VEC2, npharm_max);
  DeleteArray(&D1, npharm1);
  DeleteArray(&D2, npharm2);  
  delete[] U;
  delete[] V;
  delete[] Row;
  delete[] Col;
  DeleteArray(&MAT, npharm_max);
  DeleteArray(&SIM, npharm_max);
}

void OverlapScore(vector<PharmacophorePoint>& PQ, vector<PharmacophorePoint>& PD, double *ShapeScore, double *ChemScore, double **MAT, double **SIM, double *U, double *V, int *Row, int *Col, double **Vec1, double **Vec2, map<int,int>& Aln, int Opt){  
  
  int npharm1=PQ.size();
  int npharm2=PD.size();
  int npharm_max =MAX(npharm1, npharm2); 
  double CordVec1[3]={0}, CordVec2[3]={0}; 
  double RMSD=0.0, T[3]={0}; double R[3][3]={{0}};
  *ShapeScore=0; // Initialize the score to 0
  *ChemScore =0; // Initialize the score to 0

  double Shape = 0.00, Chem = 0.00;
  Aln.clear();
  FillMatrix(MAT, SIM, PQ, PD, Opt);
  lap(npharm_max, MAT, Row, Col, U, V); 
  
  int k=0;
  for(int i=0; i < npharm_max; ++i){
    int j       = Row[i];    
    if(i >= npharm1){continue;}
    if(j >= npharm2){continue;}       
    if(PQ[i].includedBefore){continue;}
    if(PD[j].includedBefore){continue;}
    
    CordVec1[0]=PQ[i].point.x;
    CordVec1[1]=PQ[i].point.y;
    CordVec1[2]=PQ[i].point.z;    
    CordVec2[0]=PD[j].point.x;
    CordVec2[1]=PD[j].point.y;
    CordVec2[2]=PD[j].point.z;
    double D = distsq(CordVec1,CordVec2);   
    if(D < 6.5){     
      Vec1[k][0] = PQ[i].point.x;
      Vec1[k][1] = PQ[i].point.y;
      Vec1[k][2] = PQ[i].point.z;   
      Vec2[k][0] = PD[j].point.x;
      Vec2[k][1] = PD[j].point.y;
      Vec2[k][2] = PD[j].point.z;  
      k++;           
      if((PQ[i].hasNormal) && (PD[j].hasNormal)){
	Vec1[k][0] = PQ[i].normal.x;
        Vec1[k][1] = PQ[i].normal.y;
        Vec1[k][2] = PQ[i].normal.z;
        Vec2[k][0] = PD[j].normal.x;
        Vec2[k][1] = PD[j].normal.y;
        Vec2[k][2] = PD[j].normal.z;
        k++;		
      }       
    }    
  }
  
  if(k >2){  
    Kabsch(Vec2, Vec1, k, 1, &RMSD, T, R);   
    for(int j=0; j < npharm2; ++j){
      CordVec2[0]=PD[j].point.x;
      CordVec2[1]=PD[j].point.y;
      CordVec2[2]=PD[j].point.z;
      PD[j].point.x= (R[0][0]*CordVec2[0]) + (R[0][1]*CordVec2[1]) + (R[0][2]*CordVec2[2]) + T[0];
      PD[j].point.y= (R[1][0]*CordVec2[0]) + (R[1][1]*CordVec2[1]) + (R[1][2]*CordVec2[2]) + T[1];
      PD[j].point.z= (R[2][0]*CordVec2[0]) + (R[2][1]*CordVec2[1]) + (R[2][2]*CordVec2[2]) + T[2];       
      CordVec2[0]=PD[j].normal.x;
      CordVec2[1]=PD[j].normal.y;
      CordVec2[2]=PD[j].normal.z;
      PD[j].normal.x= (R[0][0]*CordVec2[0]) + (R[0][1]*CordVec2[1]) + (R[0][2]*CordVec2[2]) + T[0];
      PD[j].normal.y= (R[1][0]*CordVec2[0]) + (R[1][1]*CordVec2[1]) + (R[1][2]*CordVec2[2]) + T[1];
      PD[j].normal.z= (R[2][0]*CordVec2[0]) + (R[2][1]*CordVec2[1]) + (R[2][2]*CordVec2[2]) + T[2];
    }
  }
  for(int i=0; i < npharm_max; ++i){
    int j       = Row[i];   
    if(i >= npharm1){continue;}
    if(j >= npharm2){continue;}
    if(PQ[i].includedBefore){continue;}
    if(PD[j].includedBefore){continue;}     
    CordVec1[0] = PQ[i].point.x;
    CordVec1[1] = PQ[i].point.y;
    CordVec1[2] = PQ[i].point.z;
    double alphaX1 = PQ[i].alpha;     
    CordVec2[0] = PD[j].point.x;
    CordVec2[1] = PD[j].point.y;
    CordVec2[2] = PD[j].point.z;	    
    double alphaX2 = PD[j].alpha;
    double D = distsq(CordVec1, CordVec2);   
    double OShape  = 1.00, OChem = 0.0;     
    double GaussO  = (GCI2 * pow(PI/(alphaX1 + alphaX2), 1.5)) * exp(-(alphaX1 * alphaX2 * D)/(alphaX1 + alphaX2));
    //double GaussO  = 0;
    OChem = SIM[i][j];
    if(D < (2*(alphaX1+alphaX2))){
      Aln.insert(std::make_pair(i,j));
    }

    if(PQ[i].hasNormal && PD[j].hasNormal){      
      double NormalD =cosine(PQ[i].normal, PD[j].normal);  
      if(PQ[i].func >1)
	OChem  *= NormalD;
    }        
    OShape   *=  GaussO;
    OChem    *=  GaussO;
    Chem   += OChem;
    Shape  += OShape;
  }
  *ChemScore  =Chem;  
  *ShapeScore =Shape;
}


void FillMatrix(double **M, double **SIM, vector<PharmacophorePoint> PQ ,vector<PharmacophorePoint> PD, int Opt){
  // Initialize the scoring matrix
  double def=100.00;
  int npharm1=PQ.size();
  int npharm2=PD.size();
  int npharm=MAX(npharm1,npharm2);
  double Vec1[3]={0}, Vec2[3]={0};
  for(int i=0; i<npharm; ++i){
    for(int j=i; j<npharm; ++j){
      M[i][j]=def; // initializing all the cells to default value
      M[j][i]=def; // initializing all the cells to default value
    }
  }

  for(int i=0;i < npharm1;++i){
    if(PQ[i].includedBefore){continue;}
    Vec1[0]=PQ[i].point.x;
    Vec1[1]=PQ[i].point.y;
    Vec1[2]=PQ[i].point.z;    
    double alphaX1 = PQ[i].alpha;   
    for(int j=0;j < npharm2;++j){
      if(PD[j].includedBefore){continue;}
      Vec2[0]=PD[j].point.x;
      Vec2[1]=PD[j].point.y;
      Vec2[2]=PD[j].point.z;
      double alphaX2 = PD[j].alpha;      
      double D = distsq(Vec1, Vec2); 
      double OShape  = 1.00, OChem = 0.00;
      double GaussO  =0;      
      GaussO  = (GCI2 * pow(PI/(alphaX1 + alphaX2), 1.5)) * exp(-(alphaX1 * alphaX2 * D)/(alphaX1 + alphaX2));           
      OChem = SIM[i][j];
      if(PQ[i].hasNormal && PD[j].hasNormal){
        double NormalD =cosine(PQ[i].normal, PD[j].normal);
        if(PQ[i].func >1)
          OChem  *= NormalD;
      }
      OShape  *=  GaussO;
      OChem   *=  GaussO;        
      if(Opt == 0){
	M[i][j] -=  OShape;
      }
      if(Opt == 1){
	M[i][j] -= (OShape + OChem);
      }
      if(Opt == 2){
	M[i][j] -=  OChem;
      }
    }
  }  
}

double
Score(double ShapeS, double ChemS, double V1shape, double V2shape, double V1chem, double V2chem, int Opt){
  double S=0;
  if(Opt == 0){
    S=(ShapeS/(V1shape + V2shape - ShapeS));
  }
  if(Opt == 1){
    S=(ShapeS/(V1shape + V2shape - ShapeS)) + (ChemS/(V1chem + V2chem - ChemS)); 
  }
  if(Opt == 2){
    S=(ChemS/(V1chem + V2chem - ChemS)); 
  }  
  return S;
}

double
cosine(Coordinate& p1, Coordinate& p2){
  double c(p1.x*p2.x + p1.y*p2.y + p1.z*p2.z);
  c /= norm(p1);
  c /= norm(p2);
  return c;
}

double
norm(Coordinate& p){
  return sqrt(p.x*p.x + p.y*p.y + p.z*p.z);
}

void transformation(vector<PharmacophorePoint>& P, double R[3][3], double T[3]){    
  int npharm=P.size();
  double CordVec1[3]={0}, CordVec2[3]={0};
  for(int k=0; k < npharm; ++k){
    CordVec1[0]=P[k].point.x;
    CordVec1[1]=P[k].point.y;
    CordVec1[2]=P[k].point.z;
    P[k].point.x= (R[0][0]*CordVec1[0]) + (R[0][1]*CordVec1[1]) + (R[0][2]*CordVec1[2]) + T[0];
    P[k].point.y= (R[1][0]*CordVec1[0]) + (R[1][1]*CordVec1[1]) + (R[1][2]*CordVec1[2]) + T[1]; 
    P[k].point.z= (R[2][0]*CordVec1[0]) + (R[2][1]*CordVec1[1]) + (R[2][2]*CordVec1[2]) + T[2];	  
    CordVec2[0]=P[k].normal.x;
    CordVec2[1]=P[k].normal.y;
    CordVec2[2]=P[k].normal.z;
    P[k].normal.x = (R[0][0]*CordVec2[0]) + (R[0][1]*CordVec2[1]) + (R[0][2]*CordVec2[2]) + T[0];
    P[k].normal.y = (R[1][0]*CordVec2[0]) + (R[1][1]*CordVec2[1]) + (R[1][2]*CordVec2[2]) + T[1];
    P[k].normal.z = (R[2][0]*CordVec2[0]) + (R[2][1]*CordVec2[1]) + (R[2][2]*CordVec2[2]) + T[2];	  
  }
}
