#include <iostream>
#include <stdio.h>
#include <string>
#include <vector>
#include <ctime>
#include <fstream>
#include <iomanip> 
#include <openbabel/mol.h>
#include <openbabel/obconversion.h>
#include <openbabel/fingerprint.h>
#include "Pharmacophores.h"
#include "Smallmolecule.h"
#include "Alignment.h"
#include "RMSD.h"
#include "Common.h"


using namespace OpenBabel;

int main(int argc, char *argv[]){
  clock_t begin = clock();
  string version ("LIGSIFT (v1.2)");  
  std::cout << " ********************************************************************************************* " << std::endl
       << " * "<<version <<": an open-source tool for ligand structural alignment and virtual screening * " << std::endl
       << " * Ambrish Roy & Jeffrey Skolnick                                                            * " << std::endl
       << " * Please mail your comments to Ambrish Roy (ambrish.roy@biology.gatech.edu)                 * " << std::endl
       << " ********************************************************************************************* " << std::endl;
  
  std::string qFilename, dFilename, oFilename, sFilename, OptimInp;  
  bool vOpt= false, qOpt= false, dbOpt= false, oOpt= false, oFast= false;
  int Optim=2, sOpt=0;
  if(argc < 6){
    std::cout << std::endl
         << "Usage:" << std::endl
         << "LIGSIFT -ql   <query ligand molecules, MOL2>" << std::endl
         << "        -db   <database molecules,MOL2>" << std::endl 
         << "        -o    <Output score file>"<< std::endl
         << "Additional options:"<< std::endl
         << "        -s    <Superposed database molecules on query,MOL2>" << std::endl
         << "        -v    For verbose output" << std::endl
         << "        -opt  0(Overlap optimized based on Shape similarity)" << std::endl
         << "              1(Overlap optimized based on Shape + Chemical similarity)" << std::endl
         << "              2(Overlap optimized based on Chemical similarity:Default method)" << std::endl
         << "For example:" << std::endl << std::endl
         << "./LIGSIFT -ql ZINCXXX.mol2   -db DB.mol2   -o alignment_scores.txt   -s superposed.mol2" << std::endl << std::endl;
    return 1;
  }
  
  for(int i = 0; i < argc; i++ ){
    if(!strcmp(argv[i],"-ql")  && i < argc ) { qFilename     = string(argv[i+1]); qOpt = true;}
    if(!strcmp(argv[i],"-QL")  && i < argc ) { qFilename     = string(argv[i+1]); qOpt = true;}
    if(!strcmp(argv[i],"-db") && i < argc ) { dFilename      = string(argv[i+1]); dbOpt= true;}
    if(!strcmp(argv[i],"-DB") && i < argc ) { dFilename      = string(argv[i+1]); dbOpt= true;}
    if(!strcmp(argv[i],"-o")  && i < argc ) { oFilename      = string(argv[i+1]); oOpt = true;}
    if(!strcmp(argv[i],"-O")  && i < argc ) { oFilename      = string(argv[i+1]); oOpt = true;}
    if(!strcmp(argv[i],"-s")  && i < argc ) { sFilename      = string(argv[i+1]); sOpt = 1;}
    if(!strcmp(argv[i],"-f")  && i < argc ) { oFast          = true;}
    if(!strcmp(argv[i],"-opt")&& i < argc ) { OptimInp       = string(argv[i+1]); Optim= atoi(OptimInp.c_str());}
    if(!strcmp(argv[i],"-v")  && i < argc ) { vOpt           = true;}
    if(!strcmp(argv[i],"-V")  && i < argc ) { vOpt           = true;}
  }  
  if (!qOpt){
    std::cout << "Please provide query molecules (preferably in MOL2 format)" << std::endl;
    exit(EXIT_FAILURE);
  }
  if(!dbOpt){
    std::cout << "Please provide database molecules (preferably in MOL2 format)" << std::endl;
    exit(EXIT_FAILURE);
  }
  if((!vOpt) && (!oOpt)){
    std::cout << "Please provide a filename for recording alignment scores." << std::endl;
    exit(EXIT_FAILURE);
  }
  if(Optim >2){Optim=2;}

  std::stringstream SS;
  std::ofstream ofs1;
  std::ifstream ifile;
  OpenBabel::OBMol qL, dL, dS, dS2;
  SmallMolecule SMq, SMd;
  Pharmacophore QPharm, TPharm;
  std::vector<OpenBabel::OBMol> DBmol;
  std::vector<PharmacophorePoint> QPharmacophores;
  std::vector<PharmacophorePoint> DPharmacophores;
  std::vector<PharmacophorePoint> PH;
  std::vector<std::string> Qnames, Dnames;
  std::vector<std::string> Qfpt, Dfpt;
  std::vector<unsigned int> fp;
  int Nmolq=0, Nmold=0, Nsup=0;  
  string line;

  //##############Start with query Ligand file###############//
  ifile.open(qFilename.c_str());
  if(!ifile.is_open()){
    std::cout << "ERROR: " << std::endl
	      << "Cannot open query template ligands list " << qFilename << std::endl;
    exit(EXIT_FAILURE);
  }
  else{
    OpenBabel::OBConversion* SMreader = new OpenBabel::OBConversion();
    OpenBabel::OBFingerprint* lFP = OpenBabel::OBFingerprint::FindFingerprint("FP2");
    bool success = SMreader->SetInFormat(SMreader->FormatFromExt(qFilename.c_str()));
    if(success){
      bool notatend = SMreader->ReadFile(&qL,qFilename);
      while(notatend){
        string MolID = qL.GetTitle(); 
        lFP->GetFingerprint(&qL,fp);
        string FPT = ConvertFingerprint(fp);
        Qfpt.push_back(FPT);
	SMq.getMoleculeData(qL);
        SMq.InitPharmacophore(qL,PH);
        QPharm.push_back(PH);
        Qnames.push_back(MolID);
        SMq.clear_molecule(); 
        ++Nmolq;
        qL.Clear(); PH.clear(); SS.str("");fp.clear();
        notatend = SMreader->Read(&qL);
      }
    }
    delete SMreader;
  }  
  //##############Now read database molecules###############//
  std::ifstream dFile(dFilename.c_str());
  if( !dFile.good() ){
    std::cout << "ERROR: " << std::endl
	      << "Cannot open database file " << dFilename << std::endl;
    exit(EXIT_FAILURE);
  } 
  else{   
    OpenBabel::OBConversion* SMreader = new OpenBabel::OBConversion();
    OpenBabel::OBFingerprint* lFP = OpenBabel::OBFingerprint::FindFingerprint("FP2");
    bool success = SMreader->SetInFormat(SMreader->FormatFromExt(dFilename.c_str()));   
    if(success){     
      bool notatend2 = SMreader->ReadFile(&dL,dFilename);  
      while(notatend2){
        if(dL.Empty())
	  continue;
	string MolID = dL.GetTitle();   
        lFP->GetFingerprint(&dL,fp);
        string FPT = ConvertFingerprint(fp);
        Dfpt.push_back(FPT);
	DBmol.push_back(dL);
	Dnames.push_back(MolID);	
	++Nmold; 
	dL.Clear();SS.str("");fp.clear();
	notatend2 = SMreader->Read(&dL);
      }   
    }
    delete SMreader;        
  }

  
  if(vOpt){
    std::cout << "No. of molecules read from query file : "<< Nmolq << std::endl;
    std::cout << "No. of molecules read from database   : "<< Nmold << std::endl;  
    clock_t initial = clock();
    double elapsed_secs = double(initial - begin)/CLOCKS_PER_SEC;
    std::cout << "Time spent in initialization: " << elapsed_secs << " seconds\n";
    if(Optim==2){
      std::cout << "Using -opt 2 (default) Chemical similarity for finding the best overlap." << std::endl;
    }
    if(Optim==0){
      std::cout << "Using -opt 0 Shape similarity for finding the best overlap." << std::endl;
    }
    if(Optim==1){
      std::cout << "Using -opt 1 combination of Shape & Chemical similarity (1:1) for finding the best overlap." << std::endl;
    }   
  }
 

  
  FILE * oFILE;
  std::map<int,int> BAlignment;
  std::vector<double> ScoreArr;
  std::vector<double> Volumes;
  ScoreArr.resize(4);
  Volumes.resize(2); 
  double **VECQ, **VECD;
  CreateArray(&VECQ,200, 3); CreateArray(&VECD,200, 3);
  oFILE= fopen(oFilename.c_str(),"w");
  if(vOpt){
    printf("#Molecule1       Molecule2        sTCshape     sTCchem   TCshape   TCchem    Pval_shape     Pval_chem     TverskyShape  TverskyChem  TC2D\n");
  }
  fprintf(oFILE,"#Molecule1       Molecule2        sTCshape     sTCchem   TCshape   TCchem    Pval_shape     Pval_chem     TverskyShape  TverskyChem  TC2D\n");
  for(unsigned int j=0;j<DBmol.size();++j){
    dS = DBmol[j];
    SMd.getMoleculeData(dS);
    SMd.InitPharmacophore(dS,PH);         
    for(unsigned int i=0;i<QPharm.size();++i){
      QPharmacophores=QPharm[i];
      DPharmacophores=PH;	
      ScoreArr[0]=0;ScoreArr[1]=0;ScoreArr[2]=0; ScoreArr[3]=0;ScoreArr[4]=0;
      Volumes[0] =0; Volumes[1]=0;      	
      Nsup++;
      Overlap(QPharmacophores,DPharmacophores,ScoreArr, Volumes, BAlignment, Optim,sOpt);
      double v1  = Volumes[0];
      double v2  = Volumes[1];

      double sTCshape=0.0, sTCchem=0.0, pval_shape=1.00, pval_chem=1.0;
      double TCshape =ScoreArr[0];
      double TCchem  =ScoreArr[1];
      double TC2D    = calc2DTC(Qfpt[i],Dfpt[j]); 
      PVal(TCshape, v1, v2, 0, Optim, &sTCshape, &pval_shape);
      PVal(TCchem,  v1, v2, 1, Optim, &sTCchem,  &pval_chem);
      if(vOpt){
	printf ("%-15s  %-15s   %-.3f         %-.3f    %-.3f    %-.3f    %e   %e      %-.3f         %-.3f     %-.3f\n",Dnames[j].c_str(), Qnames[i].c_str(), sTCshape, sTCchem, TCshape, TCchem, pval_shape, pval_chem, ScoreArr[2], ScoreArr[3], TC2D);
      }
      fprintf (oFILE,"%-15s  %-15s   %-.3f         %-.3f    %-.3f    %-.3f    %e   %e      %-.3f         %-.3f     %-.3f\n",Dnames[j].c_str(), Qnames[i].c_str(), sTCshape, sTCchem, TCshape, TCchem, pval_shape, pval_chem, ScoreArr[2], ScoreArr[3], TC2D);      
      if(sOpt==1){	
	int L1= QPharmacophores.size(); int L2=DPharmacophores.size();
	if((L1>0) && (L2>0)){
	  double RMSD=0, T[3]={0}, R[3][3]={{0}},CordVec[3]={0}; int pno=0;
	  for (std::map<int,int>::iterator it=BAlignment.begin(); it!=BAlignment.end(); ++it){	  
	    int iQ=it->first;
	    int iD=it->second;
	    VECQ[pno][0]=QPharm[i][iQ].point.x;       VECQ[pno][1]=QPharm[i][iQ].point.y; VECQ[pno][2]=QPharm[i][iQ].point.z; 
	    VECD[pno][0]=PH[iD].point.x;              VECD[pno][1]=PH[iD].point.y;        VECD[pno][2]=PH[iD].point.z;
	    ++pno;	  
	  }
	  Kabsch(VECD, VECQ, pno, 1, &RMSD, T, R);
	  dS2=DBmol[j];	  
	  FOR_ATOMS_OF_MOL(atom,dS2){   
	    CordVec[0]=atom->x();
	    CordVec[1]=atom->y();
	    CordVec[2]=atom->z();    
	    double nX=transform(T,R,CordVec,0);
	    double nY=transform(T,R,CordVec,1);    
	    double nZ=transform(T,R,CordVec,2);	  	   
	    atom->SetVector(nX,nY,nZ);	  
	  } 
	  OpenBabel::OBConversion outM;
	  outM.SetOutFormat(outM.FormatFromExt(sFilename.c_str()));
	  ofs1.open(sFilename.c_str(),ios::app);
	  outM.Write(&dS2,&ofs1);
	  ofs1.close();
	  dS2.Clear();
	}
      }      
      BAlignment.clear();
    }
    PH.clear(); 
    SMd.clear_molecule();
    dS.Clear();
  }
  fclose(oFILE);
  DeleteArray(&VECQ,200);
  DeleteArray(&VECD,200);
  for(unsigned int i = 0; i < DBmol.size(); i++){
    DBmol[i].Clear();
  }
  DBmol.empty();

  clock_t end = clock();
  double total_time = double(end - begin)/CLOCKS_PER_SEC;
  std::cout << Nsup << " structure comparisons in " << total_time << " seconds\n";
  return 0;
}


